package PScm::Env;

use strict;
use warnings;
use base qw(PScm);
use PScm::Continuation;

sub new {
    my ($class, %bindings) = @_;

    $class = ref($class) || $class;
    bless { bindings => {%bindings}, parent => 0 }, $class;
}

sub Extend {
    my ($self, $ra_symbols, $ra_values, $cont) = @_;

    PScm::Expr::List->new(@$ra_values)->map_eval(
        $self,
        cont {
            my ($ra_evaluated_values) = @_;
            $cont->Cont(
                $self->ExtendUnevaluated(
                    $ra_symbols, $ra_evaluated_values
                )
            );
        }
    );
}

sub ExtendUnevaluated {
    my ($self, $ra_symbols, $ra_values) = @_;

    my %newbindings;
    @newbindings{ map { $_->value } @$ra_symbols } = @$ra_values;
    my $newenv = $self->new(%newbindings);
    $newenv->{parent} = $self;
    return $newenv;
}

sub LookUp {
    my ($self, $symbol) = @_;

    if (exists($self->{bindings}{ $symbol->value })) {
        return $self->{bindings}{ $symbol->value };
    } elsif ($self->{parent}) {
        return $self->{parent}->LookUp($symbol);
    } else {
        die "no binding for @{[$symbol->value]} ",
          "in @{[ref($self)]}\n";
    }
}

1;
