package PScm::SpecialForm;

use strict;
use warnings;
use base qw(PScm::Expr);

###############################
package PScm::SpecialForm::Let;

use base qw(PScm::SpecialForm);
use PScm::Continuation;

sub Apply {
    my ($self, $form, $env, $cont) = @_;

    my ($bindings, $body) = $form->value;
    my (@symbols, @values);

    foreach my $binding ($bindings->value) {
        my ($symbol, $value) = $binding->value;
        push @symbols, $symbol;
        push @values,  $value;
    }

    $env->Extend(
        \@symbols,
        \@values,
        cont {
            my ($newenv) = @_;
            $body->Eval($newenv, $cont);
        }
    );
}

##############################
package PScm::SpecialForm::If;

use base qw(PScm::SpecialForm);
use PScm::Continuation;

sub Apply {
    my ($self, $form, $env, $cont) = @_;

    my ($condition, $true_branch, $false_branch) = $form->value;

    $condition->Eval(
        $env,
        cont {
            my ($result) = @_;
            if ($result->isTrue) {
                $true_branch->Eval($env, $cont);
            } else {
                $false_branch->Eval($env, $cont);
            }
        }
    );
}

##################################
package PScm::SpecialForm::Lambda;

use base qw(PScm::SpecialForm);
use PScm::Closure;

sub Apply {
    my ($self, $form, $env, $cont) = @_;

    my ($args, $body) = $form->value;

    $cont->Cont(PScm::Closure::Function->new($args, $body, $env));
}

##################################
package PScm::SpecialForm::CallCC;

use base qw(PScm::SpecialForm);
use PScm::Closure;
use PScm::Continuation;

sub Apply {
    my ($self, $form, $env, $cont) = @_;

    $form->first->Eval(
        $env,
        cont {
            my ($closure) = @_;
            $closure->Apply(new PScm::Expr::List($cont),
                $env, $cont);
        }
    );
}

1;
